#include <cmath>
#include <cstring>
#include <stdexcept>
#include "data_streamer.hpp"
#include "float80.hpp"

using namespace sc::io;

data_streamer::data_streamer(byte_order bo) : _target_byte_order(bo) {}

byte_order data_streamer::cur_byte_order() {
	uint8_t ch[2] = {0x12, 0x34};
	uint16_t* p = reinterpret_cast<uint16_t*>(ch);
	return *p == 0x1234 ? byte_order::big_endian : byte_order::little_endian;
}

std::string data_streamer::get4chars(std::istream& file) const {
	char buf[4];
	file.read(buf, 4);
	return std::string(buf, 4);
}

uint32_t data_streamer::getui24(std::istream& file) const {
	char buf[4] = {0};
	int i = cur_byte_order() == byte_order::little_endian ? 0 : 1;
	file.read(&buf[i], 3);
	if (cur_byte_order() != _target_byte_order) {
		char tmp = buf[i];
		buf[i] = buf[i + 2];
		buf[i + 2] = tmp;
	}
	return *reinterpret_cast<uint32_t*>(buf);
}

int32_t data_streamer::getsi24(std::istream& file) const {
	return static_cast<int32_t>(getui24(file));
}

double data_streamer::getf80(std::istream& file) const {
	float_80 val;
	file.read(reinterpret_cast<char*>(val._val), sizeof(val._val));
	if (_target_byte_order != byte_order::big_endian) {
		byte_swap(val._val, sizeof(val._val));
	}
	return val;
}

void data_streamer::put4chars(const std::string& s, std::ostream& file) const {
	if (s.size() < 4) {
		throw std::runtime_error("data_streamer::put4chars: string too short");
	}
	file.write(s.c_str(), 4);
}

void data_streamer::putui24(uint32_t val, std::ostream& file) const {
	char* buf = reinterpret_cast<char*>(&val);
	int i = cur_byte_order() == byte_order::little_endian ? 0 : 1;
	if (cur_byte_order() != _target_byte_order) {
		char tmp = buf[i];
		buf[i] = buf[i + 2];
		buf[i + 2] = tmp;
	}
	file.write(&buf[i], 3);
}

void data_streamer::putsi24(int32_t val, std::ostream& file) const {
	putui24(static_cast<uint32_t>(val), file);
}

void data_streamer::putf80(double val, std::ostream& file) const {
	float_80 f80 {val};
	if (_target_byte_order != byte_order::big_endian) {
		byte_swap(f80._val, sizeof(f80._val));
	}
	file.write(reinterpret_cast<char*>(f80._val), sizeof(f80._val));
}
