#ifndef FDOSTREAM_H_
#define FDOSTREAM_H_

#include <iostream>
#include "fdiobuf.hpp" // @exclude

namespace sc {
    namespace io {
        class fdostream : public std::ostream {
            public:
                fdostream() : std::ostream(&_outbuf), _outbuf(-1) {}
                fdostream(int fd) : std::ostream(&_outbuf), _outbuf(fd) {}

                // no copying
                fdostream(const fdostream&) = delete;
                fdostream& operator=(const fdostream&) = delete;

                // moving allowed
                fdostream(fdostream&& other);
                fdostream& operator=(fdostream&& other);

                // cleanup: fdostream is RAII class for open file descriptor
                virtual ~fdostream();

                int fd() const { return _outbuf.fd(); }
                void fd(int fd) { _outbuf.fd(fd); }

                bool is_open() const { return _outbuf.fd() != -1; }
                void close();

            protected:
                fdiobuf _outbuf;
        };
    }
}

#endif // FDOSTREAM_H_
