#ifndef FDSTREAM_H_
#define FDSTREAM_H_

#include <iostream>
#include "fdiobuf.hpp" // @exclude

namespace sc {
    namespace io {
        class fdstream : public std::iostream {
            public:
                fdstream() : std::iostream(&_iobuf) {}
                fdstream(int fd) : std::iostream(&_iobuf), _iobuf(fd) {}

                // no copying
                fdstream(const fdstream&) = delete;
                fdstream& operator=(const fdstream&) = delete;

                // moving allowed
                fdstream(fdstream&& other);
                fdstream& operator=(fdstream&& other);

                // cleanup: fdstream is RAII class for open file descriptor
                virtual ~fdstream();

                int fd() const { return _iobuf.fd(); }
                void fd(int fd) { _iobuf.fd(fd); }

                bool is_open() const { return _iobuf.fd() != -1; }
                void close();

            protected:
                fdiobuf _iobuf {-1};
        };
    }
}

#endif // FDSTREAM_H_
