#ifndef IMEMFILE_H_
#define IMEMFILE_H_

#include <iostream>
#include "mapped_file.hpp" // @exclude
#include "memstreambuf.hpp" // @exclude

namespace sc {
    namespace io {
        class imemfile : public std::istream {
            public:
                imemfile() : std::istream(&_buf) {}
                imemfile(const std::string& path);

                // no copying
                imemfile(const imemfile&) = delete;
                imemfile& operator=(const imemfile&) = delete;

                // move is allowed
                imemfile(imemfile&& other);
                imemfile& operator=(imemfile&&) = default;

                ~imemfile() = default;

                bool is_open() const { return _file.is_open(); }
                void open(const std::string& path, std::ios_base::openmode mode
                          = std::ios_base::in | std::ios_base::binary);

                char* data() const { return _file.mapping(); }
                size_t size() const { return _file.size(); }

            private:
                mapped_file _file;
                memstreambuf _buf;
        };
    }
}

#endif // IMEMFILE_H_
