#ifndef MAPPED_FILE_H_
#define MAPPED_FILE_H_

#include <string>

namespace sc {
    namespace io {
        class mapped_file {
            public:
                mapped_file() = default;
                mapped_file(const std::string& path);

                // no copying
                mapped_file(const mapped_file&) = delete;
                mapped_file& operator=(const mapped_file&) = delete;

                // moving allowed
                mapped_file(mapped_file&& other);
                mapped_file& operator=(mapped_file&& other);

                // cleanup: mapped_file is RAII class for memory mapping and file descriptor
                ~mapped_file();

                // it's an error to open an already opened mapped_file
                // open() is meant to be used after default constructor
                void open(const std::string& path);
                bool is_open() const { return _fd != -1; }

                // direct (read-only) access to mapped memory
                char* mapping() const { return _mapping; }
                size_t size() const { return _length; }

            private:
                int _fd {-1};
                char* _mapping {nullptr};
                size_t _length {0};
        };
    }
}

#endif // MAPPED_FILE_H_
