#ifndef MEMSTREAMBUF_H_
#define MEMSTREAMBUF_H_

#include <streambuf>
#include <iostream>

namespace sc {
    namespace io {
        class memstreambuf : public std::streambuf {
            public:
                memstreambuf() = default;
                void init(char* mem, size_t size);

                memstreambuf(char* mem, size_t size);

                void mem(char* m) { _mem = m; }
                char* mem() const { return _mem; }
                void size(size_t sz) { _size = sz; }
                size_t size() const { return _size; }

            protected:
                // input
                int_type underflow() override;
                int_type uflow() override;

                // output
                int_type overflow(int_type c) override;
                std::streamsize xsputn(const char* s, std::streamsize num) override;

                // stream offset positioning
                pos_type seekoff(off_type off,
                                 std::ios_base::seekdir dir,
                                 std::ios_base::openmode which
                                 = std::ios_base::in | std::ios_base::out) override;
                pos_type seekpos(pos_type pos, std::ios_base::openmode which
                                 = std::ios_base::in | std::ios_base::out) override;
                int_type pbackfail(int_type c = EOF) override;

            private:
                char* _mem {nullptr};
                size_t _size {0};
        };
    }
}

#endif // MEMSTREAMBUF_H_
