#ifndef SOCKETSTREAM_H_
#define SOCKETSTREAM_H_

#include <sys/socket.h>
#include <iostream>
#include "fdiobuf.hpp" // @exclude

namespace sc {
    namespace io {
        struct socket_address {
            std::string address;
            int port;
        };

        class socketstream : public std::iostream {
            public:
                socketstream();
                socketstream(int domain, int type);

                // copying forbidden
                socketstream(const socketstream&) = delete;
                socketstream& operator=(const socketstream&) = delete;

                // moving allowed
                socketstream(socketstream&& other);
                socketstream& operator=(socketstream&& other);

                // cleanup: socketstream is RAII class for open socket
                virtual ~socketstream();
                bool is_open() { return _iobuf.fd() != -1; }
                void close();

                // server
                void make_server(const socket_address& sa);
                socketstream accept() const;

                // client
                void connect(const socket_address& host);

                // convenience: human-readable addresses
                std::string address() const;

                // address/endpoint conversion methods
                void endpoint_from_address(const socket_address& address,
                                           struct sockaddr_storage* endpoint) const;
                socket_address address_from_endpoint(const struct sockaddr* endpoint) const;

                // getters & setters
                int socket() const { return _iobuf.fd(); }
                void socket(int sock) { _iobuf.fd(sock); }
                int domain() const { return _domain; }
                void domain(int dom) { _domain = dom; }
                int type() const { return _type; }
                void type(int typ) { _type = typ; }

            protected:
                fdiobuf _iobuf {-1};
                int _domain {0};
                int _type {0};
                struct sockaddr_storage _sa;
        };
    }
}

#endif // SOCKETSTREAM_H_
